@@ -1,35 +1,36 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 /*!
 # SixtyFPS
 
 This crate is the main entry point for embedding user interfaces designed with
 [SixtyFPS UI](https://sixtyfps.io/) in Rust programs.
 
-Included in this documentation is also the [language reference](docs::langref),
-documentation of [builtin elements](docs::builtin_elements), [widgets](docs::widgets) and [layouting](docs::layouting).
+If you are new to SixtyFPS, start with the [Walk-through tutorial](https://sixtyfps.io/docs/tutorial/rust).
+If you are already familiar with SixtyFPS, the following topics provide some
 
-If you are new to SixtyFPS, you might also consider going through our [Walk-through tutorial](https://sixtyfps.io/docs/tutorial/rust).
+## Foundation fopics
 
-## How to use:
+ * [The `.60` language reference](docs::langref)
+ * [Builtin Elements](docs::builtin_elements)
+ * [Widgets](docs::widgets)
+ * [Positioning and Layout of Elements](docs::layouting)
+ * [Debugging Techniques](docs::debugging_techniques)
 
-The user interfaces are described in the `.60` design markup language. There are two ways
-of including the design in Rust:
+## How to use this crate:
 
- - The `.60` code is inline in a macro.
- - The `.60` code in external files compiled with `build.rs`
+Designs of user interfaces are described in the `.60` design markup language. There are three ways
+of including them in Rust:
 
- This markup code is translated to Rust code and each component is turned into a Rust
- struct with functions to instantiated, show or access properties. This documentation
- includes an [example](docs::generated_code::SampleComponent) of how the API looks
- like.
+ - The `.60` code is [inline in a macro](#the-60-code-in-a-macro).
+ - The `.60` code in [external files compiled with `build.rs`](#the-60-code-in-external-files-is-compiled-with-buildrs)
+ - The `.60` code is loaded dynamically at run-time from the file system, by using the [interpreter API](https://docs.rs/sixtyfps-interpreter/latest/sixtyfps_interpreter/).
+
+With the first two methods, the markup code is translated to Rust code and each component is turned into a Rust
+struct with functions. Use these functions to instantiate and show the component, and
+to access declared properties. Check out our [sample component](docs::generated_code::SampleComponent) for more
+information about the generation functions and how to use them.
 
 ### The .60 code in a macro
 
@@ -50,14 +51,14 @@ fn main() {
 }
 ```
 
-### The .60 file in external files compiled with `build.rs`
+### The .60 code in external files is compiled with `build.rs`
 
-This method allows you to a separate `.60` file on the file system, which works well if
-your design becomes bigger and you split it up across multiple files. You need to use a
-so-called [build script](https://doc.rust-lang.org/cargo/reference/build-scripts.html)
-to trigger the compilation of the `.60` file.
+When your design becomes bigger in terms of markup code, you may want move it to a dedicated
+`.60` file. It's also possible to split a `.60` file into multiple files using (modules)[docs::langref#modules].
+Use a [build script](https://doc.rust-lang.org/cargo/reference/build-scripts.html) to compile
+your main `.60` file:
 
-In your Cargo.toml:
+In your Cargo.toml add a `build` assignment and use the `sixtyfps-build` crate in `build-dependencies`:
 
 ```toml
 [package]
@@ -67,14 +68,14 @@ resolver = "2" # avoid dependency conflicts on some platforms
 edition = "2018"
 
 [dependencies]
-sixtyfps = "0.1.4"
+sixtyfps = "0.1.6"
 ...
 
 [build-dependencies]
-sixtyfps-build = "0.1.4"
+sixtyfps-build = "0.1.6"
 ```
 
-In the `build.rs` file:
+Use the API of the sixtyfps-build crate in the `build.rs` file:
 
 ```ignore
 fn main() {
@@ -82,7 +83,7 @@ fn main() {
 }
 ```
 
-Then in your main file
+Finally, use the [`include_modules!`] macro in your `main.rs`:
 
 ```ignore
 sixtyfps::include_modules!();
@@ -91,8 +92,9 @@ fn main() {
 }
 ```
 
-To quickly get started with this pattern, we have a [Template Repository](https://github.com/sixtyfpsui/sixtyfps-rust-template) that
-can be used with cargo-generate.
+The [cargo-generate](https://github.com/cargo-generate/cargo-generate) tool is a great tool to up and running quickly with a new
+Rust project. You can use it in combination with our [Template Repository](https://github.com/sixtyfpsui/sixtyfps-rust-template) to
+create a skeleton file hierarchy that uses this method:
 
 ```bash
 cargo install cargo-generate
@@ -101,18 +103,30 @@ cargo generate --git https://github.com/sixtyfpsui/sixtyfps-rust-template
 
 ## Generated components
 
-As of now, only the last component of a .60 source is generated. It is planned to generate all exported components.
+Currently, only the last component in a `.60` source file is mapped to a Rust structure that be instantiated. We are tracking the
+resolution of this limitation in <https://github.com/sixtyfpsui/sixtyfps/issues/784>.
 
-The component is generated and re-exported at the location of the [`include_modules!`]  or [`sixtyfps!`] macro.
-it consist of a struct of the same name of the component.
-For example, if you have `export MyComponent := Window { /*...*/ }` in the .60 file, it will create a `struct MyComponent{ /*...*/ }`.
-This documentation contains a documented generated component: [`docs::generated_code::SampleComponent`].
+The component is generated and re-exported to the location of the [`include_modules!`]  or [`sixtyfps!`] macro. It is represented
+as a struct with the same name as the component.
 
-The component is created using the [`fn new() -> Self`](docs::generated_code::SampleComponent::new) function. In addition
-the following convenience functions are available through the [`ComponentHandle`] implementation:
+For example, if you have
 
-  - [`fn clone_strong(&self) -> Self`](docs::generated_code::SampleComponent::clone_strong): to create a strongly referenced clone.
-  - [`fn as_weak(&self) -> Weak`](docs::generated_code::SampleComponent::as_weak): to create a [weak](Weak) reference.
+```60
+export MyComponent := Window { /*...*/ }
+```
+
+in the .60 file, it will create a
+```rust
+struct MyComponent{ /*...*/ }
+```
+
+See also our [sample component](docs::generated_code::SampleComponent) for more information about the API of the generated struct.
+
+A component is instantiated using the [`fn new() -> Self`](docs::generated_code::SampleComponent::new) function. The following
+convenience functions are available through the [`ComponentHandle`] implementation:
+
+  - [`fn clone_strong(&self) -> Self`](docs::generated_code::SampleComponent::clone_strong): creates a strongly referenced clone of the component instance.
+  - [`fn as_weak(&self) -> Weak`](docs::generated_code::SampleComponent::as_weak): to create a [weak](Weak) reference to the component instance.
   - [`fn show(&self)`](docs::generated_code::SampleComponent::show): to show the window of the component.
   - [`fn hide(&self)`](docs::generated_code::SampleComponent::hide): to hide the window of the component.
   - [`fn run(&self)`](docs::generated_code::SampleComponent::run): a convenience function that first calls `show()`,
@@ -127,27 +141,26 @@ For each top-level callback
   - [`fn invoke_<callback_name>(&self)`](docs::generated_code::SampleComponent::invoke_hello): to invoke the callback
   - [`fn on_<callback_name>(&self, callback: impl Fn(<CallbackArgs>) + 'static)`](docs::generated_code::SampleComponent::on_hello): to set the callback handler.
 
-Note that all dashes (`-`) are replaced by underscores (`_`) in names of types or functions.
+Note: All dashes (`-`) are replaced by underscores (`_`) in names of types or functions.
 
-After instantiating the component you can call just [`ComponentHandle::run()`] on it, in order to show it and spin the event loop to
-render and react to input events. If you want to show multiple components simultaneously, then you can also call just
-[`ComponentHandle::show()`] first. When you're ready to enter the event loop, just call [`run_event_loop()`].
+After instantiating the component, call [`ComponentHandle::run()`] on show it on the screen and spin the event loop to
+react to input events. To show multiple components simultaneously, call [`ComponentHandle::show()`] on each instance.
+Call [`run_event_loop()`] when you're ready to enter the event loop.
 
-The generated component struct act as a handle holding a strong reference (similar to a `Rc`). It does not implement
-`Clone` because we want to make explicit if we are cloning a strong reference (with [`ComponentHandle::clone_strong`]),
-or a weak reference (with  [`ComponentHandle::as_weak`]). A strong reference should not be captured by the closures
-given to a callback, as this would produce a reference loop and leak the component. Instead, the callback function
-should capture a weak component.
+The generated component struct acts as a handle holding a strong reference (similar to an `Rc`). The `Clone` trait is
+not implemented. Instead you need to make explicit [`ComponentHandle::clone_strong`] and [`ComponentHandle::as_weak`]
+calls. A strong reference should not be captured by the closures given to a callback, as this would produce a reference
+loop and leak the component. Instead, the callback function should capture a weak component.
 
 ## Threading and Event-loop
 
 For platform-specific reasons, the event loop must run in the main thread, in most backends, and all the components
 must be created in the same thread as the thread the event loop is running or is going to run.
 
-Ideally, you should perform the minimum amount of work in the main thread and delegate the actual logic to another
-thread. To communicate from your worker thread to the UI thread, the [`invoke_from_event_loop`] function can be used.
+You should perform the minimum amount of work in the main thread and delegate the actual logic to another
+thread to avoid blocking animations. Use the [`invoke_from_event_loop`] function to communicate from your worker thread to the UI thread.
 
-To run a function with a delay or with an interval you can use a [`Timer`].
+To run a function with a delay or with an interval use a [`Timer`].
 
 ## Type Mappings
 
@@ -212,7 +225,8 @@ See the [documentation of the `Global` trait](Global) for an example.
 #![warn(missing_docs)]
 #![deny(unsafe_code)]
 #![doc(html_logo_url = "https://sixtyfps.io/resources/logo.drawio.svg")]
-
+#![cfg_attr(not(feature = "std"), no_std)]
+extern crate alloc;
 pub use sixtyfps_macros::sixtyfps;
 
 pub use sixtyfps_corelib::graphics::{
@@ -229,6 +243,7 @@ pub use sixtyfps_corelib::timers::{Timer, TimerMode};
 /// for use with the `font-family` property. The provided slice must be a valid TrueType
 /// font.
 #[doc(hidden)]
+#[cfg(feature = "std")]
 pub fn register_font_from_memory(data: &'static [u8]) -> Result<(), Box<dyn std::error::Error>> {
     sixtyfps_rendering_backend_default::backend().register_font_from_memory(data)
 }
@@ -237,6 +252,7 @@ pub fn register_font_from_memory(data: &'static [u8]) -> Result<(), Box<dyn std:
 /// for use with the `font-family` property. The provided path must refer to a valid TrueType
 /// font.
 #[doc(hidden)]
+#[cfg(feature = "std")]
 pub fn register_font_from_path<P: AsRef<std::path::Path>>(
     path: P,
 ) -> Result<(), Box<dyn std::error::Error>> {
@@ -246,21 +262,23 @@ pub fn register_font_from_path<P: AsRef<std::path::Path>>(
 /// internal re_exports used by the macro generated
 #[doc(hidden)]
 pub mod re_exports {
+    pub use alloc::boxed::Box;
+    pub use alloc::format;
+    pub use alloc::rc::{Rc, Weak};
+    pub use alloc::string::String;
+    pub use alloc::{vec, vec::Vec};
     pub use const_field_offset::{self, FieldOffsets, PinnedDrop};
     pub use core::iter::FromIterator;
-    pub use once_cell::sync::Lazy;
+    pub use once_cell::race::OnceBox;
     pub use once_cell::unsync::OnceCell;
     pub use pin_weak::rc::PinWeak;
     pub use sixtyfps_corelib::animations::EasingCurve;
     pub use sixtyfps_corelib::callbacks::Callback;
     pub use sixtyfps_corelib::component::{
-        init_component_items, Component, ComponentRefPin, ComponentVTable,
-    };
-    pub use sixtyfps_corelib::graphics::{
-        Brush, GradientStop, Image, ImageInner, LinearGradientBrush, PathArcTo, PathCubicTo,
-        PathData, PathElement, PathEvent, PathLineTo, PathMoveTo, PathQuadraticTo, Point, Rect,
-        Size,
+        free_component_item_graphics_resources, init_component_items, Component, ComponentRefPin,
+        ComponentVTable,
     };
+    pub use sixtyfps_corelib::graphics::*;
     pub use sixtyfps_corelib::input::{
         FocusEvent, InputEventResult, KeyEvent, KeyEventResult, KeyboardModifiers, MouseEvent,
     };
@@ -287,8 +305,8 @@ pub mod re_exports {
 #[doc(hidden)]
 pub mod internal {
     use crate::re_exports::*;
+    use alloc::rc::Rc;
     use core::pin::Pin;
-    use std::rc::Rc;
 
     // Helper functions called from generated code to reduce code bloat from
     // extra copies of the original functions for each call site due to
@@ -310,6 +328,16 @@ pub mod internal {
         }
     }
 
+    impl<C: 'static> StrongComponentRef for VRcMapped<ComponentVTable, C> {
+        type Weak = VWeakMapped<ComponentVTable, C>;
+        fn to_weak(&self) -> Self::Weak {
+            VRcMapped::downgrade(self)
+        }
+        fn from_weak(weak: &Self::Weak) -> Option<Self> {
+            weak.upgrade()
+        }
+    }
+
     impl<C: 'static> StrongComponentRef for Pin<Rc<C>> {
         type Weak = PinWeak<C>;
         fn to_weak(&self) -> Self::Weak {
@@ -448,7 +476,7 @@ pub fn quit_event_loop() {
 /// handle.run();
 /// ```
 pub fn invoke_from_event_loop(func: impl FnOnce() + Send + 'static) {
-    sixtyfps_rendering_backend_default::backend().post_event(Box::new(func))
+    sixtyfps_rendering_backend_default::backend().post_event(alloc::boxed::Box::new(func))
 }
 
 /// This trait is used to obtain references to global singletons exported in `.60`
@@ -503,6 +531,7 @@ pub trait ComponentHandle {
         Self: Sized;
 
     /// Returns a clone of this handle that's a strong reference.
+    #[must_use]
     fn clone_strong(&self) -> Self;
 
     /// Internal function used when upgrading a weak reference to a strong one.
@@ -553,19 +582,28 @@ mod weak_handle {
     /// This is useful to use with [`invoke_from_event_loop()`] or [`Self::upgrade_in_event_loop()`].
     pub struct Weak<T: ComponentHandle> {
         inner: vtable::VWeak<re_exports::ComponentVTable, T::Inner>,
+        #[cfg(feature = "std")]
         thread: std::thread::ThreadId,
     }
 
     impl<T: ComponentHandle> Clone for Weak<T> {
         fn clone(&self) -> Self {
-            Self { inner: self.inner.clone(), thread: self.thread }
+            Self {
+                inner: self.inner.clone(),
+                #[cfg(feature = "std")]
+                thread: self.thread,
+            }
         }
     }
 
     impl<T: ComponentHandle> Weak<T> {
         #[doc(hidden)]
         pub fn new(rc: &vtable::VRc<re_exports::ComponentVTable, T::Inner>) -> Self {
-            Self { inner: vtable::VRc::downgrade(rc), thread: std::thread::current().id() }
+            Self {
+                inner: vtable::VRc::downgrade(rc),
+                #[cfg(feature = "std")]
+                thread: std::thread::current().id(),
+            }
         }
 
         /// Returns a new strongly referenced component if some other instance still
@@ -577,6 +615,7 @@ mod weak_handle {
         where
             T: ComponentHandle,
         {
+            #[cfg(feature = "std")]
             if std::thread::current().id() != self.thread {
                 return None;
             }
@@ -614,6 +653,7 @@ mod weak_handle {
         /// # thread.join().unwrap(); return; // don't run the event loop in examples
         /// handle.run();
         /// ```
+        #[cfg(feature = "std")]
         pub fn upgrade_in_event_loop(self, func: impl FnOnce(T) + Send + 'static)
         where
             T: 'static,
@@ -629,6 +669,7 @@ mod weak_handle {
     // Safety: we make sure in upgrade that the thread is the proper one,
     // and the VWeak only use atomic pointer so it is safe to clone and drop in another thread
     #[allow(unsafe_code)]
+    #[cfg(feature = "std")]
     unsafe impl<T: ComponentHandle> Send for Weak<T> {}
 }
 
@@ -637,6 +678,7 @@ pub use weak_handle::*;
 pub use sixtyfps_corelib::window::api::Window;
 
 /// This module contains functions useful for unit tests
+#[cfg(feature = "std")]
 pub mod testing {
     use core::cell::Cell;
     thread_local!(static KEYBOARD_MODIFIERS : Cell<crate::re_exports::KeyboardModifiers> = Default::default());
@@ -725,7 +767,7 @@ macro_rules! include_modules {
 /// Helper type that helps checking that the generated code is generated for the right version
 #[doc(hidden)]
 #[allow(non_camel_case_types)]
-pub struct VersionCheck_0_1_4;
+pub struct VersionCheck_0_1_6;
 
 #[cfg(doctest)]
 mod compile_fail_tests;
